@@ -9,7 +9,7 @@ from gi.repository import GLib, Gtk
 
 from xpra.util import (
     CLIENT_EXIT,
-    iround, envbool,
+    envbool, u,
     ellipsizer, repr_ellipsized, reverse_dict, typedict,
     )
 from xpra.os_util import bytestostr, OSX, WIN32
@@ -71,12 +71,7 @@ CLIPBOARD_DIRECTION_LABEL_TO_NAME = {
                                      }
 CLIPBOARD_DIRECTION_NAME_TO_LABEL = reverse_dict(CLIPBOARD_DIRECTION_LABEL_TO_NAME)
 
-
-def s(v):
-    try:
-        return v.decode("utf-8")
-    except AttributeError:
-        return str(v)
+SERVER_NOT_SUPPORTED = "Not supported by the server"
 
 
 class GTKTrayMenuBase(MenuHelper):
@@ -140,6 +135,8 @@ class GTKTrayMenuBase(MenuHelper):
         if SHOW_VERSION_CHECK:
             menu.append(self.make_updatecheckmenuitem())
         menu.append(self.make_bugreportmenuitem())
+        menu.append(self.make_docsmenuitem())
+        menu.append(self.make_html5menuitem())
         info_menu_item.show_all()
         return info_menu_item
 
@@ -672,7 +669,7 @@ class GTKTrayMenuBase(MenuHelper):
     def make_scalingvaluemenuitem(self, scaling_submenu, scalingvalue=1.0):
         def scalecmp(v):
             return abs(self.client.xscale-v)<0.1
-        pct = iround(100.0*scalingvalue)
+        pct = round(100.0*scalingvalue)
         label = {100 : "None"}.get(pct, "%i%%" % pct)
         c = Gtk.CheckMenuItem(label=label)
         c.scalingvalue = scalingvalue
@@ -903,7 +900,7 @@ class GTKTrayMenuBase(MenuHelper):
             c.set_active(current_value==delta)
             def activate_cb(item, *_args):
                 avsynclog("activate_cb(%s, %s) delta=%s", item, menu, delta)
-                if delta==0:
+                if delta is None:
                     self.client.av_sync = False
                     self.client.send_sound_sync(0)
                 else:
@@ -1166,6 +1163,7 @@ class GTKTrayMenuBase(MenuHelper):
         menu = Gtk.Menu()
         windows_menu_item.set_submenu(menu)
         menu.append(self.make_raisewindowsmenuitem())
+        menu.append(self.make_showhidewindowsmenuitem())
         menu.append(self.make_minimizewindowsmenuitem())
         menu.append(self.make_refreshmenuitem())
         menu.append(self.make_reinitmenuitem())
@@ -1186,21 +1184,68 @@ class GTKTrayMenuBase(MenuHelper):
             self.client.reinit_window_icons()
         return self.handshake_menuitem("Re-initialize", "reinitialize.png", None, force_reinit)
 
+    def _non_OR_windows(self):
+        return tuple(win for win in self.client._window_to_id.keys() if not win.is_OR())
+
+    def _call_non_OR_windows(self, functions):
+        for win in self._non_OR_windows():
+            for function, args in functions.items():
+                fn = getattr(win, function, None)
+                if not fn:
+                    log.warn("Warning: no '%s' function on %s", function, win)
+                else:
+                    try:
+                        fn(*args)
+                    except Exception as e:
+                        log.error("Error calling %s%s on %s:", function, args, win)
+                        log.error(" %s", e)
+
+    def _raise_all_windows(self, *_args):
+        self._call_non_OR_windows({"deiconify" : (), "present" : ()})
+
     def make_raisewindowsmenuitem(self):
-        def raise_windows(*_args):
-            for win in self.client._window_to_id:
-                if not win.is_OR():
-                    win.deiconify()
-                    win.present()
-        return self.handshake_menuitem("Raise Windows", "raise.png", None, raise_windows)
+        return self.handshake_menuitem("Raise Windows", "raise.png", None, self._raise_all_windows())
+
+    def _minimize_all_windows(self, *_args):
+        self._call_non_OR_windows({"iconify" : ()})
 
     def make_minimizewindowsmenuitem(self):
-        def minimize_windows(*_args):
-            for win in self.client._window_to_id:
-                if not win.is_OR():
-                    win.iconify()
-        return self.handshake_menuitem("Minimize Windows", "minimize.png", None, minimize_windows)
+        return self.handshake_menuitem("Minimize Windows", "minimize.png", None, self._minimize_all_windows)
 
+    def _hide_window(self, win):
+        skip_pager = win.get_skip_pager_hint()
+        skip_taskbar = win.get_skip_taskbar_hint()
+        def ondeiconify():
+            win.set_skip_pager_hint(skip_pager)
+            win.set_skip_taskbar_hint(skip_taskbar)
+        win._ondeiconify.append(ondeiconify)
+        if not skip_pager:
+            win.set_skip_pager_hint(True)
+        if not skip_taskbar:
+            win.set_skip_taskbar_hint(True)
+        win.freeze()
+
+    def make_showhidewindowsmenuitem(self):
+        def set_icon(icon_name):
+            image = self.get_image(icon_name, self.menu_icon_size)
+            if image:
+                self.showhidewindows.set_image(image)
+        def showhide_windows(*args):
+            self.showhidewindows_state = not self.showhidewindows_state
+            log("showhide_windows%s showhidewindows_state=%s", args, self.showhidewindows_state)
+            if self.showhidewindows_state:
+                #deiconify() will take care of restoring the attributes via "_ondeiconify"
+                self._call_non_OR_windows({"unfreeze" : (), "present" : ()})
+                set_icon("eye-off.png")
+                self.showhidewindows.set_label("Hide Windows")
+            else:
+                for win in self._non_OR_windows():
+                    self._hide_window(win)
+                set_icon("eye-on.png")
+                self.showhidewindows.set_label("Show Windows")
+        self.showhidewindows_state = True
+        self.showhidewindows = self.handshake_menuitem("Hide Windows", "eye-off.png", None, showhide_windows)
+        return self.showhidewindows
 
     def make_servermenuitem(self):
         server_menu_item = self.handshake_menuitem("Server", "server.png")
@@ -1231,7 +1276,7 @@ class GTKTrayMenuBase(MenuHelper):
             log("enable_servercommands%s server-commands-info=%s", args, self.client.server_commands_info)
             set_sensitive(self.servercommands, self.client.server_commands_info)
             if not self.client.server_commands_info:
-                self.servercommands.set_tooltip_text("Not supported by the server")
+                self.servercommands.set_tooltip_text(SERVER_NOT_SUPPORTED)
             else:
                 self.servercommands.set_tooltip_text("")
         self.client.after_handshake(enable_servercommands)
@@ -1272,7 +1317,7 @@ class GTKTrayMenuBase(MenuHelper):
             log("enable_upload%s server_file_transfer=%s", args, self.client.remote_file_transfer)
             set_sensitive(self.upload, self.client.remote_file_transfer)
             if not self.client.remote_file_transfer:
-                self.upload.set_tooltip_text("Not supported by the server")
+                self.upload.set_tooltip_text(SERVER_NOT_SUPPORTED)
             else:
                 self.upload.set_tooltip_text("Send a file to the server")
         self.client.after_handshake(enable_upload)
@@ -1285,7 +1330,7 @@ class GTKTrayMenuBase(MenuHelper):
                 args, self.client.remote_file_transfer, self.client.server_start_new_commands, self.client._remote_subcommands)
             set_sensitive(self.download, self.client.remote_file_transfer)
             if not self.client.remote_file_transfer or not self.client.server_start_new_commands:
-                self.download.set_tooltip_text("Not supported by the server")
+                self.download.set_tooltip_text(SERVER_NOT_SUPPORTED)
             elif "send-file" not in self.client._remote_subcommands:
                 self.download.set_tooltip_text("'send-file' subcommand is not supported by the server")
             else:
@@ -1302,7 +1347,7 @@ class GTKTrayMenuBase(MenuHelper):
             log("enable_download%s server_file_transfer=%s", args, self.client.remote_file_transfer)
             set_sensitive(self.download_log, self.client.remote_file_transfer and bool(self.client._remote_server_log))
             if not self.client.remote_file_transfer:
-                self.download_log.set_tooltip_text("Not supported by the server")
+                self.download_log.set_tooltip_text(SERVER_NOT_SUPPORTED)
             elif not self.client._remote_server_log:
                 self.download_log.set_tooltip_text("Server does not expose its log-file")
             else:
@@ -1375,7 +1420,7 @@ class GTKTrayMenuBase(MenuHelper):
         execlog("build_start_menu() %i menu items", len(self.client.server_xdg_menu))
         execlog("self.client.server_xdg_menu=%s", ellipsizer(self.client.server_xdg_menu))
         for cat, category_props in sorted(self.client.server_xdg_menu.items()):
-            category = s(cat)
+            category = u(cat)
             execlog(" * category: %s", category)
             #log("category_props(%s)=%s", category, category_props)
             if not isinstance(category_props, dict):
@@ -1393,7 +1438,7 @@ class GTKTrayMenuBase(MenuHelper):
             category_menu_item.set_submenu(cat_menu)
             menu.append(category_menu_item)
             for an, cp in sorted(entries.items()):
-                app_name = s(an)
+                app_name = u(an)
                 command_props = typedict(cp)
                 execlog("  - app_name=%s", app_name)
                 app_menu_item = self.make_applaunch_menu_item(app_name, command_props)
@@ -1414,12 +1459,12 @@ class GTKTrayMenuBase(MenuHelper):
         app_menu_item = self.start_menuitem(app_name, icondata)
         def app_launch(*args):
             log("app_launch(%s) command_props=%s", args, command_props)
-            command = command_props.bytesget("command")
+            command = command_props.strget("command")
             try:
-                command = re.sub(b'\\%[fFuU]', b'', command)
+                command = re.sub('\\%[fFuU]', '', command)
             except Exception:
                 log("re substitution failed", exc_info=True)
-                command = command.split(b"%", 1)[0]
+                command = command.split("%", 1)[0]
             log("command=%s", command)
             if command:
                 self.client.send_start_command(app_name, command, False, self.client.server_sharing)
