@@ -6,11 +6,12 @@
 
 import os
 import re
+from time import monotonic
 from collections import deque
 
-from xpra.os_util import monotonic_time, POSIX
+from xpra.os_util import POSIX
 from xpra.util import envint, envbool, csv, typedict
-from xpra.exit_codes import EXIT_TIMEOUT
+from xpra.exit_codes import EXIT_CONNECTION_LOST
 from xpra.net.packet_encoding import ALL_ENCODERS
 from xpra.client.mixins.stub_client_mixin import StubClientMixin
 from xpra.scripts.config import parse_with_unit
@@ -21,6 +22,8 @@ bandwidthlog = Logger("bandwidth")
 
 FAKE_BROKEN_CONNECTION = envint("XPRA_FAKE_BROKEN_CONNECTION")
 PING_TIMEOUT = envint("XPRA_PING_TIMEOUT", 60)
+MIN_PING_TIMEOUT = envint("XPRA_MIN_PING_TIMEOUT", 2)
+MAX_PING_TIMEOUT = envint("XPRA_MAX_PING_TIMEOUT", 10)
 SWALLOW_PINGS = envbool("XPRA_SWALLOW_PINGS", False)
 #LOG_INFO_RESPONSE = ("^window.*position", "^window.*size$")
 LOG_INFO_RESPONSE = os.environ.get("XPRA_LOG_INFO_RESPONSE", "")
@@ -40,7 +43,7 @@ class NetworkState(StubClientMixin):
     """
 
     def __init__(self):
-        StubClientMixin.__init__(self)
+        super().__init__()
         self.server_start_time = -1
         #legacy:
         self.compression_level = 0
@@ -235,7 +238,7 @@ class NetworkState(StubClientMixin):
             return False
         last = self._server_ok
         if FAKE_BROKEN_CONNECTION>0:
-            self._server_ok = (int(monotonic_time()) % FAKE_BROKEN_CONNECTION) <= (FAKE_BROKEN_CONNECTION//2)
+            self._server_ok = (int(monotonic()) % FAKE_BROKEN_CONNECTION) <= (FAKE_BROKEN_CONNECTION//2)
         else:
             self._server_ok = self.last_ping_echoed_time>=ping_sent_time
         if not self._server_ok:
@@ -264,17 +267,21 @@ class NetworkState(StubClientMixin):
         log("check_echo_timeout(%s) last_ping_echoed_time=%s", ping_time, self.last_ping_echoed_time)
         if self.last_ping_echoed_time<ping_time:
             #no point trying to use disconnect_and_quit() to tell the server here..
-            self.warn_and_quit(EXIT_TIMEOUT, "server ping timeout - waited %s seconds without a response" % PING_TIMEOUT)
+            self.warn_and_quit(EXIT_CONNECTION_LOST, "server ping timeout - waited %s seconds without a response" % PING_TIMEOUT)
 
     def send_ping(self):
-        now_ms = int(1000.0*monotonic_time())
+        p = self._protocol
+        if not p or p.TYPE!="xpra":
+            self.ping_timer = None
+            return False
+        now_ms = int(1000.0*monotonic())
         self.send("ping", now_ms)
         wait = 2.0
         spl = tuple(self.server_ping_latency)
         if spl:
             spl = tuple(x[1] for x in spl)
             avg = sum(spl) / len(spl)
-            wait = min(5, 1.0+avg*2.0)
+            wait = max(MIN_PING_TIMEOUT, min(MAX_PING_TIMEOUT, 1.0+avg*2.0))
             log("send_ping() timestamp=%s, average server latency=%.1f, using max wait %.2fs",
                 now_ms, 1000.0*avg, wait)
         t = self.timeout_add(int(1000.0*wait), self.check_server_echo, now_ms)
@@ -285,11 +292,11 @@ class NetworkState(StubClientMixin):
         echoedtime, l1, l2, l3, cl = packet[1:6]
         self.last_ping_echoed_time = echoedtime
         self.check_server_echo(0)
-        server_ping_latency = monotonic_time()-echoedtime/1000.0
-        self.server_ping_latency.append((monotonic_time(), server_ping_latency))
+        server_ping_latency = monotonic()-echoedtime/1000.0
+        self.server_ping_latency.append((monotonic(), server_ping_latency))
         self.server_load = l1, l2, l3
         if cl>=0:
-            self.client_ping_latency.append((monotonic_time(), cl/1000.0))
+            self.client_ping_latency.append((monotonic(), cl/1000.0))
         log("ping echo server load=%s, measured client latency=%sms", self.server_load, cl)
 
     def _process_ping(self, packet):
@@ -320,7 +327,8 @@ class NetworkState(StubClientMixin):
         self.send_deflate_level()
 
     def send_deflate_level(self):
-        if self._protocol:
+        p = self._protocol
+        if p and p.TYPE=="xpra":
             self._protocol.set_compression_level(self.compression_level)
             self.send("set_deflate", self.compression_level)
 
