@@ -13,6 +13,7 @@ DEFAULT_BOX_COLORS = {
     "rgb32"     : "red",
     "webp"      : "pink",
     "jpeg"      : "purple",
+    "jpega"     : "plum",
     "png/P"     : "indigo",
     "png/L"     : "teal",
     "h265"      : "khaki",
@@ -29,6 +30,7 @@ BOX_COLORS = {
     "h264"      : (0.0,                 0.0,                    0.9999847412109375, ALPHA),
     "h265"      : (0.941162109375,      0.901947021484375,      0.54901123046875,   ALPHA),
     "jpeg"      : (0.501953125,         0.0,                    0.501953125,        ALPHA),
+    "jpega"     : (0.8666666666666667,  0.6274509803921569,     0.8666666666666667, ALPHA),
     "mpeg4"     : (0.0,                 0.0,                    0.0,                ALPHA),
     "png"       : (0.9999847412109375,  0.9999847412109375,     0.0,                ALPHA),
     "png/L"     : (0.0,                 0.501953125,            0.501953125,        ALPHA),
