@@ -7,20 +7,20 @@ import os
 import curses
 import signal
 import traceback
+from time import monotonic
 from subprocess import Popen, PIPE, DEVNULL
 from datetime import datetime, timedelta
 
-from xpra.version_util import caps_to_version
-from xpra.util import noerr,typedict, std, envint, csv, engs, repr_ellipsized
+from xpra.version_util import caps_to_version, full_version_str
+from xpra.util import u, noerr, typedict, std, envint, csv, engs, repr_ellipsized
 from xpra.os_util import (
     platform_name, get_machine_id,
-    bytestostr, monotonic_time,
+    bytestostr, strtobytes,
     POSIX, SIGNAMES,
     )
 from xpra.exit_codes import EXIT_STR
 from xpra.make_thread import start_thread
-from xpra.client.gobject_client_base import MonitorXpraClient
-from xpra.gtk_common.gobject_compat import register_os_signals
+from xpra.client.gobject_client_base import InfoTimerClient
 from xpra.platform.dotxpra import DotXpra
 from xpra.platform.paths import get_nodock_command
 from xpra.simple_stats import std_unit
@@ -46,7 +46,6 @@ SIGNAL_KEYS = {
 
 
 def get_title():
-    from xpra.version_util import full_version_str
     return "Xpra top %s" % full_version_str()
 
 def curses_init():
@@ -138,14 +137,14 @@ class TopClient:
     def update_loop(self):
         while self.exit_code is None:
             self.update_screen()
-            elapsed = int(1000*monotonic_time()-self.last_getch)
+            elapsed = int(1000*monotonic()-self.last_getch)
             delay = max(100, min(1000, 1000-elapsed))//100
             curses.halfdelay(delay)
             try:
                 v = self.stdscr.getch()
             except Exception:
                 v = -1
-            self.last_getch = int(1000*monotonic_time())
+            self.last_getch = int(1000*monotonic())
             if v in EXIT_KEYS:
                 self.exit_code = 0
             if v in SIGNAL_KEYS:
@@ -172,7 +171,7 @@ class TopClient:
             env["XPRA_CONNECT_TIMEOUT"] = "3"
             proc = self.do_run_subcommand("top", env=env)
             if not proc:
-                self.message = monotonic_time(), "failed to execute subprocess", curses.color_pair(RED)
+                self.message = monotonic(), "failed to execute subprocess", curses.color_pair(RED)
                 return
             exit_code = proc.wait()
             txt = "top subprocess terminated"
@@ -184,7 +183,7 @@ class TopClient:
                     txt += " (%s)" % EXIT_STR.get(exit_code, "").replace("_", " ")
                 elif (exit_code-128) in SIGNAMES:   #pylint: disable=superfluous-parens
                     txt += " (%s)" % SIGNAMES[exit_code-128]
-            self.message = monotonic_time(), txt, attr
+            self.message = monotonic(), txt, attr
         finally:
             self.setup()
 
@@ -234,7 +233,7 @@ class TopClient:
                 return
             if self.message:
                 ts, txt, attr = self.message
-                if monotonic_time()-ts<10:
+                if monotonic()-ts<10:
                     self.stdscr.addstr(hpos, 0, txt, attr)
                     hpos += 1
                     if height<=hpos:
@@ -349,62 +348,43 @@ class TopClient:
         box(self.stdscr, x, y, w, h, ul, ur, ll, lr)
 
 
-class TopSessionClient(MonitorXpraClient):
+class TopSessionClient(InfoTimerClient):
 
     def __init__(self, *args):
         super().__init__(*args)
         self.log_file = None
         if CURSES_LOG:
             self.log_file = open(CURSES_LOG, "ab")
-        self.info_request_pending = False
-        self.server_last_info = typedict()
-        self.server_last_info_time = 0
-        self.info_timer = 0
         self.paused = False
         self.stdscr = None
         self.modified = False
         self.psprocess = {}
-        self.setup()
         start_thread(self.input_thread, "input-thread", daemon=True)
 
     def client_type(self):
-        #overriden in subclasses!
         return "top"
 
     def server_connection_established(self, caps):
         self.log("server_connection_established(%s)" % repr_ellipsized(caps))
         self.log("traceback: %s" % (traceback.extract_stack(),))
-        r = super().server_connection_established(caps)
-        self.stdscr = curses_init()
+        self.setup()
         self.update_screen()
-        return r
+        return super().server_connection_established(caps)
 
     def setup(self):
-        self.stdscr = curses_init()
+        if self.stdscr is None:
+            self.stdscr = curses_init()
         try:
             curses.cbreak()
             curses.halfdelay(10)
         except Exception as e:
             self.log("failed to configure curses: %s" % e)
 
-    def run(self):
-        register_os_signals(self.signal_handler, None)
-        v = super().run()
-        self.log("run()=%s" % v)
-        self.cleanup()
-        self.close_log()
-        return v
-
-    def signal_handler(self, signum, *args):
-        self.log("exit_code=%s" % self.exit_code)
-        self.log("signal_handler(%s, %s)" % (signum, args,))
-        self.quit(128+signum)
-        self.log("exit_code=%s" % self.exit_code)
-
     def cleanup(self):
-        self.cancel_info_timer()
-        MonitorXpraClient.cleanup(self)
+        super().cleanup()
         curses_clean(self.stdscr)
+        self.stdscr = None
+        self.close_log()
 
     def close_log(self):
         log_file = self.log_file
@@ -431,6 +411,18 @@ class TopSessionClient(MonitorXpraClient):
         else:
             curses_err(self.stdscr, e)
 
+
+    def dictwarn(self, msg, *args):
+        try:
+            self.log(msg % (args,))
+        except Exception as e:
+            self.log("error logging message: %s" % e)
+    def td(self, d):
+        d = typedict(d)
+        #override warning method so we don't corrupt the curses output
+        d.warn = self.dictwarn
+        return d
+
     def update_screen(self):
         self.modified = True
 
@@ -491,15 +483,15 @@ class TopSessionClient(MonitorXpraClient):
             build = self.slidictget("server", "build")
             vstr = caps_to_version(build)
             mode = server_info.strget("mode", "server")
-            python_info = typedict(server_info.dictget("python", {}))
+            python_info = self.td(server_info.dictget("python", {}))
             bits = python_info.intget("bits", 32)
             server_str = "Xpra %s server version %s %i-bit" % (mode, vstr, bits)
             proxy_info = self.slidictget("proxy")
             if proxy_info:
-                proxy_platform_info = typedict(proxy_info.dictget("platform", {}))
+                proxy_platform_info = self.td(proxy_info.dictget("platform", {}))
                 proxy_platform = proxy_platform_info.strget("")
                 proxy_release = proxy_platform_info.strget("release")
-                proxy_build_info = typedict(proxy_info.dictget("build", {}))
+                proxy_build_info = self.td(proxy_info.dictget("build", {}))
                 proxy_version = proxy_build_info.strget("version")
                 proxy_distro = proxy_info.strget("linux_distribution")
                 server_str += " via: %s proxy version %s" % (
@@ -548,7 +540,7 @@ class TopSessionClient(MonitorXpraClient):
             cpuinfo = self.slidictget("cpuinfo")
             if cpuinfo:
                 rinfo += ", %s" % cpuinfo.strget("hz_actual")
-            elapsed = monotonic_time()-self.server_last_info_time
+            elapsed = monotonic()-self.server_last_info_time
             color = WHITE
             if self.server_last_info_time==0:
                 rinfo += " - no server data"
@@ -562,8 +554,13 @@ class TopSessionClient(MonitorXpraClient):
             dinfo = []
             server = self.slidictget("server")
             rws = server.intpair("root_window_size", None)
+            display_info = self.slidictget("display")
             if rws:
-                sinfo = "%ix%i display" % (rws[0], rws[1])
+                sinfo = "%ix%i" % (rws[0], rws[1])
+                depth = display_info.intget("depth")
+                if depth>0:
+                    sinfo += " %i-bit" % depth
+                sinfo += " display"
                 mds = server.intpair("max_desktop_size")
                 if mds:
                     sinfo += " (max %ix%i)" % (mds[0], mds[1])
@@ -572,7 +569,6 @@ class TopSessionClient(MonitorXpraClient):
             if cursor_info:
                 cx, cy = cursor_info.inttupleget("position", (0, 0))
                 dinfo.append("cursor at %ix%i" % (cx, cy))
-            display_info = self.slidictget("display")
             pid = display_info.intget("pid")
             if pid:
                 dinfo.append("pid %i" % pid)
@@ -585,34 +581,42 @@ class TopSessionClient(MonitorXpraClient):
                 addstr_main(5, 0, gl_info)
                 hpos += 1
 
+            #filter clients, only show GUI clients:
+            client_info = self.slidictget("client")
+            gui_clients = []
+            nclients = 0
+            while True:
+                ci = client_info.dictget(nclients)
+                if not ci:
+                    break
+                ci = self.td(ci)
+                session_id = ci.strget("session-id")
+                if session_id!=self.session_id and ci.boolget("windows", True) and ci.strget("type")!="top":
+                    gui_clients.append(nclients)
+                nclients += 1
+
+            ngui = 0
             if hpos<height-3:
                 hpos += 1
                 if nclients==0:
-                    addstr_main(hpos, 0, "no clients connected")
+                    clients_str = "no clients connected"
                 else:
-                    addstr_main(hpos, 0, "%i client%s connected:" % (nclients, engs(nclients)))
+                    ngui = len(gui_clients)
+                    clients_str = "%i client%s connected, " % (nclients, engs(nclients))
+                    if ngui==0:
+                        clients_str += "no gui clients"
+                    else:
+                        clients_str += "%i gui client%s:" % (ngui, engs(ngui))
+                addstr_main(hpos, 0, clients_str)
                 hpos += 1
-            client_info = self.slidictget("client")
-            client_no = 0
-            while True:
+            for client_index, client_no in enumerate(gui_clients):
                 ci = client_info.dictget(client_no)
-                if not ci:
-                    break
-                client_no +=1
-                ci = typedict(ci)
-                session_id = ci.strget("session-id")
-                if session_id:
-                    #don't show ourselves:
-                    if session_id==self.session_id:
-                        continue
-                elif not ci.boolget("windows", True):
-                    #for older servers, hide any client that doesn't display windows:
-                    continue
-                ci = self.get_client_info(ci)
+                assert ci
+                ci = self.get_client_info(self.td(ci))
                 l = len(ci)
                 if hpos+2+l>height:
                     if hpos<height:
-                        more = nclients-client_no
+                        more = ngui-client_index
                         addstr_box(hpos, 0, "%i client%s not shown" % (more, engs(more)), curses.A_BOLD)
                     break
                 self.box(1, hpos, width-2, 2+l)
@@ -630,7 +634,7 @@ class TopSessionClient(MonitorXpraClient):
             wins = tuple(windows.values())
             nwindows = len(wins)
             for win_no, win in enumerate(wins):
-                wi = self.get_window_info(typedict(win))
+                wi = self.get_window_info(self.td(win))
                 l = len(wi)
                 if hpos+2+l>height:
                     if hpos<height:
@@ -652,7 +656,7 @@ class TopSessionClient(MonitorXpraClient):
     def dictget(self, dictinstance, *parts):
         d = dictinstance
         for part in parts:
-            d = typedict(d.dictget(part, {}))
+            d = self.td(d.dictget(part, {}))
         return d
 
     def get_client_info(self, ci):
@@ -666,7 +670,7 @@ class TopSessionClient(MonitorXpraClient):
             conn_info = "connected from %s " % chost
         cinfo = ci.dictget("connection")
         if cinfo:
-            cinfo = typedict(cinfo)
+            cinfo = self.td(cinfo)
             conn_info += "using %s %s" % (cinfo.strget("type"), cinfo.strget("protocol-type"))
             conn_info += ", with %s and %s" % (cinfo.strget("encoder"), cinfo.strget("compressor"))
         gl_info = self.get_gl_info(ci.dictget("opengl"))
@@ -676,8 +680,8 @@ class TopSessionClient(MonitorXpraClient):
             audio_info.append(self._audio_info(ci, mode))
         audio_info.append(self._avsync_info(ci))
         #batch delay / latency:
-        b_info = typedict(ci.dictget("batch", {}))
-        bi_info = typedict(b_info.dictget("delay", {}))
+        b_info = self.td(ci.dictget("batch", {}))
+        bi_info = self.td(b_info.dictget("delay", {}))
         bcur = bi_info.intget("cur")
         bavg = bi_info.intget("avg")
         batch_info = "batch delay: %i (%i)" %(
@@ -696,16 +700,16 @@ class TopSessionClient(MonitorXpraClient):
             bl_color = RED
         batch_latency = batch_info.ljust(24)+"latency: %i (%i)" % (lcur, lavg)
         #speed / quality:
-        edict = typedict(ci.dictget("encoding") or {})
+        edict = self.td(ci.dictget("encoding") or {})
         qs_info = ""
         qs_color = GREEN
         if edict:
-            sinfo = typedict(edict.dictget("speed") or {})
+            sinfo = self.td(edict.dictget("speed") or {})
             if sinfo:
                 cur = sinfo.intget("cur")
                 avg = sinfo.intget("avg")
                 qs_info = "speed: %s%% (avg: %s%%)" % (cur, avg)
-            qinfo = typedict(edict.dictget("quality") or {})
+            qinfo = self.td(edict.dictget("quality") or {})
             if qinfo:
                 qs_info = qs_info.ljust(24)
                 cur = qinfo.intget("cur")
@@ -728,7 +732,7 @@ class TopSessionClient(MonitorXpraClient):
         minfo = self.dictget(ci, "sound", mode)
         if not minfo:
             return "%s off" % mode
-        minfo = typedict(minfo)
+        minfo = self.td(minfo)
         audio_info = "%s: %s" % (mode, minfo.strget("codec_description") or \
                                  minfo.strget("codec") or \
                                  minfo.strget("state", "unknown"))
@@ -765,12 +769,11 @@ class TopSessionClient(MonitorXpraClient):
         pid = wi.intget("pid", 0)
         if pid:
             line1 = "pid %i: " % pid
-        title = wi.bytesget("title", b"")
+        title = wi.get("title", "")
+        if not isinstance(title, str):
+            title = u(strtobytes(title))
         if title:
-            try:
-                line1 += ' "%s"' % title.decode("utf8")
-            except UnicodeDecodeError:
-                line1 += ' "%s"' % bytestostr(title)
+            line1 += ' "%s"' % title
         attrs = [
             x for x in (
                 "above", "below", "bypass-compositor",
@@ -793,13 +796,13 @@ class TopSessionClient(MonitorXpraClient):
     def get_gl_info(self, gli):
         if not gli:
             return None
-        gli = typedict(gli)
+        gli = self.td(gli)
         def strget(key, sep="."):
             #fugly warning:
             #depending on where we get the gl info from,
             #the value might be a list of strings,
             #or a byte string...
-            v = gli.rawget(key)
+            v = gli.get(key)
             if isinstance(v, (tuple, list)):
                 return sep.join(bytestostr(x) for x in v)
             return bytestostr(v)
@@ -809,7 +812,7 @@ class TopSessionClient(MonitorXpraClient):
         depth = gli.intget("depth")
         if depth not in (0, 24):
             gl_info += ", %ibits" % depth
-        modes = gli.rawget("display_mode")
+        modes = gli.get("display_mode")
         if modes:
             gl_info += " - %s" % strget("display_mode", ", ")
         return gl_info
@@ -818,48 +821,3 @@ class TopSessionClient(MonitorXpraClient):
         box(self.stdscr, x, y, w, h,
             ul=curses.ACS_ULCORNER, ur=curses.ACS_URCORNER,     #@UndefinedVariable
             ll=curses.ACS_LLCORNER, lr=curses.ACS_LRCORNER)     #@UndefinedVariable
-
-
-    def do_command(self, caps : typedict):
-        self.send_info_request()
-        self.timeout_add(REFRESH_RATE*1000, self.send_info_request)
-
-    def send_info_request(self):
-        self.log("send_info_request()")
-        categories = ()
-        if not self.info_request_pending:
-            self.info_request_pending = True
-            window_ids = ()    #no longer used or supported by servers
-            self.send("info-request", [self.uuid], window_ids, categories)
-        if not self.info_timer:
-            self.info_timer = self.timeout_add((REFRESH_RATE+2)*1000, self.info_timeout)
-        return True
-
-    def init_packet_handlers(self):
-        MonitorXpraClient.init_packet_handlers(self)
-        self.add_packet_handler("info-response", self._process_info_response, False)
-
-    def _process_server_event(self, packet):
-        self.log("server event: %s" % (packet,))
-        self.last_server_event = packet[1:]
-        self.update_screen()
-
-    def _process_info_response(self, packet):
-        self.log("info response: %s" % repr_ellipsized(packet))
-        self.cancel_info_timer()
-        self.info_request_pending = False
-        self.server_last_info = typedict(packet[1])
-        self.server_last_info_time = monotonic_time()
-        #log.info("server_last_info=%s", self.server_last_info)
-        self.update_screen()
-
-    def cancel_info_timer(self):
-        it = self.info_timer
-        if it:
-            self.info_timer = None
-            self.source_remove(it)
-
-    def info_timeout(self):
-        self.log("info timeout")
-        self.update_screen()
-        return True
