@@ -1,17 +1,19 @@
 # This file is part of Xpra.
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2018 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2021 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+from time import monotonic
+from collections import deque
+
 from xpra.platform.paths import get_icon_filename
 from xpra.log import Logger
-from xpra.os_util import monotonic_time
-from collections import deque
+
 log = Logger("tray")
 
 
-class TrayBase(object):
+class TrayBase:
     """
         Utility superclass for all tray implementations
     """
@@ -43,13 +45,16 @@ class TrayBase(object):
             self.tray_widget = None
 
     def ready(self):
-        pass
+        """
+        This is called when we have finished the startup sequence.
+        The MacOS dock overrides this method.
+        """
 
     def show(self):
-        raise Exception("override me!")
+        raise NotImplementedError
 
     def hide(self):
-        raise Exception("override me!")
+        raise NotImplementedError
 
     def get_screen(self):
         return -1
@@ -58,7 +63,7 @@ class TrayBase(object):
         return None     #assume "HORIZONTAL"
 
     def get_geometry(self):
-        raise Exception("override me!")
+        raise NotImplementedError
 
     def get_size(self):
         g = self.get_geometry()
@@ -68,22 +73,25 @@ class TrayBase(object):
 
     def set_tooltip(self, tooltip=None):
         self.tooltip = tooltip
-        raise Exception("override me!")
+        raise NotImplementedError
 
     def set_blinking(self, on):
-        raise Exception("override me!")
+        raise NotImplementedError
 
 
-    def set_icon_from_data(self, pixels, has_alpha, w, h, rowstride, options={}):
-        raise Exception("override me!")
+    def set_icon_from_data(self, pixels, has_alpha, w, h, rowstride, options=None):
+        raise NotImplementedError
 
     def get_icon_filename(self, basename=None):
-        return get_icon_filename(basename or self.default_icon_filename, self.default_icon_extension)
+        name = basename or self.default_icon_filename
+        f = get_icon_filename(name, self.default_icon_extension)
+        if not f:
+            log.error("Error: cannot find icon '%s'", name)
+        return f
 
     def set_icon(self, basename=None):
         filename = self.get_icon_filename(basename)
         if not filename:
-            log.error("Error: cannot find icon '%s' for name '%s'", filename, basename)
             return
         log("set_icon(%s) using filename=%s", basename, filename)
         self.set_icon_from_file(filename)
@@ -93,19 +101,20 @@ class TrayBase(object):
         if not self.tray_widget:
             return
         self.do_set_icon_from_file(filename)
-        self.icon_timestamp = monotonic_time()
+        self.icon_timestamp = monotonic()
 
     def do_set_icon_from_file(self, filename):
-        raise Exception("override me!")
+        raise NotImplementedError
 
     def recalculate_geometry(self, x, y, width, height):
-        log("recalculate_geometry%s guess=%s, tray event locations: %s", (x, y, width, height), self.geometry_guess, len(self.tray_event_locations))
+        log("recalculate_geometry%s guess=%s, tray event locations: %s",
+            (x, y, width, height), self.geometry_guess, len(self.tray_event_locations))
         if x is None or y is None:
             return
         if self.geometry_guess is None:
             #better than nothing!
             self.geometry_guess = x, y, width, height
-        if len(self.tray_event_locations)>0 and self.tray_event_locations[-1]==(x,y):
+        if self.tray_event_locations and self.tray_event_locations[-1]==(x,y):
             #unchanged
             log("tray event location unchanged")
             return
