@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2020 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2021 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -18,7 +18,7 @@ from xpra.net.net_util import get_info as get_net_info
 from xpra.child_reaper import reaper_cleanup
 from xpra.platform.info import get_sys_info
 from xpra.os_util import (
-    platform_name, bytestostr, strtobytes,
+    platform_name, bytestostr,
     BITS, POSIX, WIN32, OSX, is_Wayland,
     get_frame_info, get_info_env, get_sysconfig_info,
     )
@@ -28,10 +28,10 @@ from xpra.util import (
     XPRA_AUDIO_NOTIFICATION_ID, XPRA_DISCONNECT_NOTIFICATION_ID,
     )
 from xpra.scripts.config import parse_bool
-from xpra.exit_codes import EXIT_CONNECTION_FAILED, EXIT_OK, EXIT_CONNECTION_LOST
+from xpra.exit_codes import EXIT_CONNECTION_FAILED, EXIT_CONNECTION_LOST
 from xpra.version_util import get_version_info_full, get_platform_info
 from xpra.client import mixin_features
-from xpra.log import Logger
+from xpra.log import Logger, get_info as get_log_info
 
 
 CLIENT_BASES = [XpraClientBase]
@@ -278,6 +278,7 @@ class UIXpraClient(ClientBaseClass):
             "env"       : get_info_env(),
             "sys"       : get_sys_info(),
             "network"   : get_net_info(),
+            "logging"   : get_log_info(),
             }
         if SYSCONFIG:
             info["sysconfig"] = get_sysconfig_info()
@@ -363,6 +364,8 @@ class UIXpraClient(ClientBaseClass):
         self.may_notify(XPRA_DISCONNECT_NOTIFICATION_ID,
                         "Xpra Session Disconnected: %s" % reason, body, icon_name="disconnected")
         delay = NOTIFICATION_EXIT_DELAY*mixin_features.notifications
+        if self.exit_code is None:
+            self.exit_code = self.server_disconnect_exit_code(reason, *info)
         self.timeout_add(delay*1000, XpraClientBase.server_disconnect, self, reason, *info)
         self.cleanup()
 
@@ -377,8 +380,9 @@ class UIXpraClient(ClientBaseClass):
         if POSIX and not is_Wayland():
             #we may be running inside another server!
             try:
-                from xpra.server.server_uuid import get_uuid
-                caps["server_uuid"] = get_uuid() or ""
+                from xpra.server.server_uuid import get_uuid, get_mode  #pylint: disable=import-outside-toplevel
+                if get_mode()!="shadow":
+                    caps["server_uuid"] = get_uuid() or ""
             except ImportError:
                 pass
         for x in (#generic feature flags:
@@ -426,7 +430,7 @@ class UIXpraClient(ClientBaseClass):
             if not cb.parse_server_capabilities(self, c):
                 log.info("failed to parse server capabilities in %s", cb)
                 return False
-        self.server_session_name = strtobytes(c.rawget("session_name", b"")).decode("utf-8")
+        self.server_session_name = c.uget("session_name")
         set_name("Xpra", self.session_name or self.server_session_name or "Xpra")
         self.server_platform = c.strget("platform")
         self.server_sharing = c.boolget("sharing")
@@ -507,7 +511,7 @@ class UIXpraClient(ClientBaseClass):
 
     def _process_startup_complete(self, packet):
         log("all the existing windows and system trays have been received")
-        XpraClientBase._process_startup_complete(self, packet)
+        super()._process_startup_complete(packet)
         gui_ready()
         if self.tray:
             self.tray.ready()
@@ -590,7 +594,7 @@ class UIXpraClient(ClientBaseClass):
 
 
     def get_control_commands_caps(self):
-        caps = ["show_session_info", "show_bug_report", "debug"]
+        caps = ["show_session_info", "show_bug_report", "show_menu", "name", "debug"]
         for x in compression.get_enabled_compressors():
             caps.append("enable_"+x)
         for x in packet_encoding.get_enabled_encoders():
@@ -607,6 +611,8 @@ class UIXpraClient(ClientBaseClass):
             self.show_session_info(*args)
         elif command=="show_bug_report":
             self.show_bug_report()
+        elif command=="show_menu":
+            self.show_menu()
         elif command in ("enable_%s" % x for x in compression.get_enabled_compressors()):
             compressor = command.split("_")[1]
             log.info("switching to %s on server request", compressor)
