@@ -147,7 +147,9 @@ def xpra_codec_import(name, description, top_module, class_module, classname):
 
 CODEC_OPTIONS = {
     #encoders:
+    "enc_rgb"       : ("RGB encoder",       "argb",         "encoder", "encode"),
     "enc_pillow"    : ("Pillow encoder",    "pillow",       "encoder", "encode"),
+    "enc_spng"      : ("png encoder",       "spng",         "encoder", "encode"),
     "enc_webp"      : ("webp encoder",      "webp",         "encoder", "encode"),
     "enc_jpeg"      : ("JPEG encoder",      "jpeg",         "encoder", "encode"),
     "enc_nvjpeg"    : ("nvjpeg encoder",    "nvjpeg",       "encoder", "encode"),
@@ -163,6 +165,7 @@ CODEC_OPTIONS = {
     "csc_cython"    : ("cython colorspace conversion", "csc_cython", "colorspace_converter", "ColorspaceConverter"),
     #decoders:
     "dec_pillow"    : ("Pillow decoder",    "pillow",       "decoder", "decompress"),
+    "dec_spng"      : ("png decoder",       "spng",         "decoder", "decompress"),
     "dec_webp"      : ("webp decoder",      "webp",         "decoder", "decompress"),
     "dec_jpeg"      : ("JPEG decoder",      "jpeg",         "decoder", "decompress_to_rgb", "decompress_to_yuv"),
     #video decoders:
@@ -185,37 +188,37 @@ def load_codec(name):
 
 
 def load_codecs(encoders=True, decoders=True, csc=True, video=True):
-    show = []
     log("loading codecs")
 
+    loaded = []
     def load(*names):
         for name in names:
+            if has_codec(name):
+                continue
             load_codec(name)
+            if has_codec(name) and name not in loaded:
+                loaded.append(name)
 
     if encoders:
-        show += list(ENCODER_CODECS)
         load(*ENCODER_CODECS)
         if video:
-            show += list(ENCODER_VIDEO_CODECS)
             load(*ENCODER_VIDEO_CODECS)
     if csc and video:
-        show += list(CSC_CODECS)
         load(*CSC_CODECS)
     if decoders:
-        show += list(DECODER_CODECS)
         load(*DECODER_CODECS)
         if video:
-            show += list(DECODER_VIDEO_CODECS)
             load(*DECODER_VIDEO_CODECS)
-    log("done loading codecs")
-    show_codecs()
+    log("done loading codecs: %s", loaded)
+    return loaded
 
-def show_codecs():
+def show_codecs(show=None):
     #print("codec_status=%s" % codecs)
-    for name in sorted(ALL_CODECS):
+    for name in sorted(show or ALL_CODECS):
         log("* %s : %s %s" % (name.ljust(20), str(name in codecs).ljust(10), codecs.get(name, "")))
     log("codecs versions:")
-    for name, version in codec_versions.items():
+    for name in (show or codec_versions.keys()):
+        version = codec_versions.get(name, "")
         log("* %s : %s" % (name.ljust(20), version))
 
 
@@ -233,9 +236,9 @@ def has_codec(name) -> bool:
 
 
 CSC_CODECS = "csc_swscale", "csc_cython", "csc_libyuv"
-ENCODER_CODECS = "enc_pillow", "enc_webp", "enc_jpeg", "enc_nvjpeg"
+ENCODER_CODECS = "enc_rgb", "enc_pillow", "enc_spng", "enc_webp", "enc_jpeg", "enc_nvjpeg"
 ENCODER_VIDEO_CODECS = "enc_vpx", "enc_x264", "enc_x265", "nvenc", "enc_ffmpeg"
-DECODER_CODECS = "dec_pillow", "dec_webp", "dec_jpeg"
+DECODER_CODECS = "dec_pillow", "dec_spng", "dec_webp", "dec_jpeg"
 DECODER_VIDEO_CODECS = "dec_vpx", "dec_avcodec2"
 
 ALL_CODECS = tuple(set(CSC_CODECS + ENCODER_CODECS + ENCODER_VIDEO_CODECS + DECODER_CODECS + DECODER_VIDEO_CODECS))
@@ -263,6 +266,7 @@ def get_encoding_name(encoding):
           "png/P"   : "PNG (8bpp colour)",
           "png/L"   : "PNG (8bpp grayscale)",
           "jpeg"    : "JPEG",
+          "jpega"   : "JPEG with alpha",
           "rgb"     : " + ".join(get_rgb_compression_options()) + " (24/32bpp)",
         }
     return ENCODINGS_TO_NAME.get(encoding, encoding)
@@ -315,7 +319,7 @@ def main(args):
         if verbose:
             format_string = LOG_FORMAT
             log.enable_debug()
-        enable_color(format_string)
+        enable_color(format_string=format_string)
 
         if len(args)>1:
             for x in args[1:]:
@@ -324,9 +328,8 @@ def main(args):
         else:
             load_codecs()
             list_codecs = ALL_CODECS
-
-        #not really a codec, but gets used by codecs, so include version info:
-        add_codec_version("numpy", "numpy")
+            #not really a codec, but gets used by codecs, so include version info:
+            add_codec_version("numpy", "numpy")
         print("codecs and csc modules found:")
         #print("codec_status=%s" % codecs)
         for name in sorted(list_codecs):
