@@ -26,6 +26,8 @@ CODEC_TO_MODULE = {
     "libyuv"     : "csc_libyuv",
     "avcodec2"   : "dec_avcodec2",
     "ffmpeg"     : "enc_ffmpeg",
+    "jpeg"       : "jpeg",
+    "nvjpeg"     : "nvjpeg",
     }
 
 def has_codec_module(module_name):
@@ -48,8 +50,8 @@ def try_import_modules(*codec_names):
 
 #all the codecs we know about:
 #try to import the module that contains them (cheap check):
-ALL_VIDEO_ENCODER_OPTIONS = try_import_modules("x264", "vpx", "x265", "nvenc", "ffmpeg")
-HARDWARE_ENCODER_OPTIONS = try_import_modules("nvenc")
+ALL_VIDEO_ENCODER_OPTIONS = try_import_modules("x264", "vpx", "x265", "nvenc", "ffmpeg", "nvjpeg", "jpeg")
+HARDWARE_ENCODER_OPTIONS = try_import_modules("nvenc", "nvjpeg")
 ALL_CSC_MODULE_OPTIONS = try_import_modules("swscale", "cython", "libyuv")
 NO_GFX_CSC_OPTIONS = []
 ALL_VIDEO_DECODER_OPTIONS = try_import_modules("avcodec2", "vpx")
@@ -144,7 +146,7 @@ class VideoHelper:
             else:
                 notfound = tuple(x for x in (exclist+inclist) if x and x not in all_list)
                 if notfound:
-                    log.warn("Warning: ignoring unknown %s: %s", name, csv(notfound))    
+                    log.warn("Warning: ignoring unknown %s: %s", name, csv(notfound))
             return tuple(x for x in inclist if x not in exclist)
         self.video_encoders = filt("video encoders" , video_encoders,   ALL_VIDEO_ENCODER_OPTIONS)
         self.csc_modules    = filt("csc modules"    , csc_modules,      ALL_CSC_MODULE_OPTIONS)
@@ -433,7 +435,7 @@ def getVideoHelper():
 
 
 def main():
-    from xpra.codecs.loader import log as loader_log, load_codecs
+    from xpra.codecs.loader import log as loader_log, load_codecs, show_codecs
     from xpra.util import print_nested_dict
     from xpra.log import enable_color
     from xpra.platform import program_context
@@ -443,6 +445,7 @@ def main():
             loader_log.enable_debug()
             log.enable_debug()
         load_codecs()
+        show_codecs()
         vh = getVideoHelper()
         vh.set_modules(ALL_VIDEO_ENCODER_OPTIONS, ALL_CSC_MODULE_OPTIONS, ALL_VIDEO_DECODER_OPTIONS)
         vh.init()
