@@ -27,6 +27,10 @@ EXIT_NO_DISPLAY = 20
 EXIT_SERVER_ALREADY_EXISTS = 21
 EXIT_SOCKET_CREATION_ERROR = 22
 EXIT_VFB_ERROR = 23
+EXIT_FILE_NOT_FOUND = 24
+EXIT_UPGRADE = 25
+EXIT_IO_ERROR = 26
+EXIT_NO_DATA = 27
 
 
 EXIT_STR = {
@@ -54,4 +58,14 @@ EXIT_STR = {
     EXIT_SERVER_ALREADY_EXISTS  : "SERVER_ALREADY_EXISTS",
     EXIT_SOCKET_CREATION_ERROR  : "SOCKET_CREATION_ERROR",
     EXIT_VFB_ERROR              : "VFB_ERROR",
+    EXIT_FILE_NOT_FOUND         : "FILE_NOT_FOUND",
+    EXIT_UPGRADE                : "UPGRADE",
+    EXIT_IO_ERROR               : "IO_ERROR",
+    EXIT_NO_DATA                : "NO_DATA",
     }
+
+RETRY_EXIT_CODES = [
+    EXIT_CONNECTION_LOST,
+    EXIT_PACKET_FAILURE,
+    EXIT_UPGRADE,
+    ]
