@@ -1,8 +1,9 @@
 # This file is part of Xpra.
-# Copyright (C) 2016-2019 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2016-2021 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+import sys
 from xpra.util import envbool
 
 XPRA_MDNS_TYPE = "_xpra._tcp."
@@ -23,13 +24,19 @@ def get_listener_class():
             from xpra.net.mdns.avahi_listener import AvahiListener
             log("AvahiListener=%s", AvahiListener)
             return AvahiListener
-        except ImportError as e:
-            log("failed to import AvahiListener: %s", e)
+        except ImportError:
+            log("failed to import AvahiListener", exc_info=True)
     if ZEROCONF:
+        #workaround for MacOS Big Sur which broke ctypes,
+        #ctypes is used in the ifaddr module which is imported by zeroconf:
+        if sys.platform.startswith("darwin"):
+            import xpra.platform    #pylint: disable=import-outside-toplevel
+            #on MacOS, an import side-effect is to patch the ctypes loader
+            assert xpra.platform
         try:
             from xpra.net.mdns.zeroconf_listener import ZeroconfListener
             log("ZeroconfListener=%s", ZeroconfListener)
             return ZeroconfListener
-        except ImportError as e:
-            log("failed to import ZeroconfListener: %s", e)
+        except (ImportError, OSError):
+            log("failed to import ZeroconfListener", exc_info=True)
     return None
