@@ -148,6 +148,7 @@ def do_get_bind_ifacemask(iface):
                     except Exception as e:
                         log("do_get_bind_ifacemask(%s)", iface, exc_info=True)
                         log.error("Error converting address '%s' to binary, for interface %s", addr, iface)
+                        log.error(" %s", e)
     log("do_get_bind_ifacemask(%s)=%s", iface, ipmasks)
     return ipmasks
 
@@ -482,6 +483,10 @@ def main(): # pragma: no cover
 
         print("")
         print("Protocol Capabilities:")
+        from xpra.net import compression
+        compression.init_all()
+        from xpra.net import packet_encoding
+        packet_encoding.init_all()
         netcaps = get_network_caps()
         netif = {""    : bool(netifaces)}
         if netifaces_version:
