@@ -1,16 +1,16 @@
 # This file is part of Xpra.
-# Copyright (C) 2014-2019 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2014-2021 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-from xpra.server.auth.sys_auth_base import SysAuthenticator, log
-from xpra.platform.info import get_username
+from xpra.server.auth.sys_auth_base import SysAuthenticator
+from xpra.util import typedict
 
 
 class Authenticator(SysAuthenticator):
 
-    def __init__(self, username, **kwargs):
-        super().__init__(username or get_username(), **kwargs)
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
         self.salt = None
 
     def requires_challenge(self) -> bool:
@@ -22,7 +22,7 @@ class Authenticator(SysAuthenticator):
     def get_password(self):
         return None
 
-    def authenticate(self, _challenge_response=None, _client_salt=None) -> bool:
+    def authenticate(self, caps : typedict) -> bool:
         return True
 
     def __repr__(self):
