@@ -14,8 +14,8 @@ from xpra.server.auth.sqlauthbase import SQLAuthenticator, DatabaseUtilBase, run
 
 class Authenticator(SQLAuthenticator):
 
-    def __init__(self, username, filename="sqlite.sdb", **kwargs):
-        super().__init__(username, **kwargs)
+    def __init__(self, filename="sqlite.sdb", **kwargs):
+        super().__init__(**kwargs)
         if filename and not os.path.isabs(filename):
             exec_cwd = kwargs.get("exec_cwd", os.getcwd())
             filename = os.path.join(exec_cwd, filename)
