@@ -1,11 +1,10 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2010-2020 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2021 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-from xpra.os_util import strtobytes
-from xpra.util import engs, iround, log_screen_sizes
+from xpra.util import engs, log_screen_sizes
 from xpra.os_util import bytestostr
 from xpra.scripts.config import FALSE_OPTIONS
 from xpra.server.mixins.stub_server_mixin import StubServerMixin
@@ -40,6 +39,7 @@ class DisplayManager(StubServerMixin):
         self.bell = opts.bell
         self.cursors = opts.cursors
         self.default_dpi = int(opts.dpi)
+        self.bit_depth = self.get_display_bit_depth()
 
 
     def parse_hello(self, ss, caps, send_ui):
@@ -140,6 +140,7 @@ class DisplayManager(StubServerMixin):
                     "y"         : self.ydpi,
                     },
                 "antialias" : self.antialias,
+                "depth" : self.bit_depth,
                 }
         if self.opengl_props:
             i["opengl"] = self.opengl_props
@@ -288,7 +289,7 @@ class DisplayManager(StubServerMixin):
             if xdpi!=self.xdpi or ydpi!=self.ydpi:
                 self.xdpi, self.ydpi = xdpi, ydpi
                 log("new dpi: %ix%i", self.xdpi, self.ydpi)
-                self.dpi = iround((self.xdpi + self.ydpi)/2.0)
+                self.dpi = round((self.xdpi + self.ydpi)/2)
                 self.dpi_changed()
         if len(packet)>=8:
             #added in 0.16 for scaled client displays:
@@ -304,48 +305,21 @@ class DisplayManager(StubServerMixin):
         self.set_screen_size(width, height, bigger)
         if len(packet)>=4:
             log.info("received updated display dimensions")
-            log.info("client display size is %sx%s with %s screen%s:",
-                     width, height, len(ss.screen_sizes), engs(ss.screen_sizes))
+            log.info("client display size is %sx%s",
+                     width, height)
             log_screen_sizes(width, height, ss.screen_sizes)
             self.calculate_workarea(width, height)
         #ensures that DPI and antialias information gets reset:
         self.update_all_server_settings()
 
     def dpi_changed(self):
-        pass
+        """
+        The x11 servers override this method
+        to also update the XSettings.
+        """
 
     def calculate_desktops(self):
-        count = 1
-        for ss in self._server_sources.values():
-            if ss.desktops:
-                count = max(count, ss.desktops)
-        count = max(1, min(20, count))
-        names = []
-        for i in range(count):
-            if i==0:
-                name = "Main"
-            else:
-                name = "Desktop %i" % (i+1)
-            for ss in self._server_sources.values():
-                if ss.desktops and i<len(ss.desktop_names) and ss.desktop_names[i]:
-                    dn = ss.desktop_names[i]
-                    if isinstance(dn, str):
-                        #newer clients send unicode
-                        name = dn
-                    else:
-                        #older clients send byte strings:
-                        try :
-                            v = strtobytes(dn).decode("utf8")
-                        except (UnicodeEncodeError, UnicodeDecodeError):
-                            log.error("Error parsing '%s'", dn, exc_info=True)
-                        else:
-                            if v!="0" or i!=0:
-                                name = v
-            names.append(name)
-        self.set_desktops(names)
-
-    def set_desktops(self, names):
-        pass
+        """ seamless servers can update the desktops """
 
     def calculate_workarea(self, w, h):
         raise NotImplementedError()
