@@ -1,13 +1,15 @@
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2010-2018 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2021 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 #pylint: disable-msg=E1101
 
-from xpra.os_util import monotonic_time, bytestostr
-from xpra.util import typedict
+from time import monotonic
+
+from xpra.os_util import bytestostr
+from xpra.util import typedict, net_utf8
 from xpra.server.mixins.stub_server_mixin import StubServerMixin
 from xpra.log import Logger
 
@@ -39,6 +41,8 @@ class InputServer(StubServerMixin):
         #timers for cancelling key repeat when we get jitter
         self.key_repeat_timer = None
 
+        self.last_mouse_user = None
+
     def setup(self):
         self.watch_keymap_changes()
 
@@ -74,7 +78,7 @@ class InputServer(StubServerMixin):
             self.parse_hello_ui_keyboard(ss, caps)
 
     def watch_keymap_changes(self):
-        pass
+        """ GTK servers will start listening for the 'keys-changed' signal """
 
     def parse_hello_ui_keyboard(self, ss, c):
         other_ui_clients = [s.uuid for s in self._server_sources.values() if s!=ss and s.ui_client]
@@ -94,7 +98,7 @@ class InputServer(StubServerMixin):
         self.set_keymap(ss)
 
     def get_keyboard_info(self) -> dict:
-        start = monotonic_time()
+        start = monotonic()
         info = {
              "repeat"           : {
                                    "delay"      : self.key_repeat_delay,
@@ -106,7 +110,7 @@ class InputServer(StubServerMixin):
         kc = self.keyboard_config
         if kc:
             info.update(kc.get_info())
-        keylog("get_keyboard_info took %ims", (monotonic_time()-start)*1000)
+        keylog("get_keyboard_info took %ims", (monotonic()-start)*1000)
         return info
 
 
@@ -152,12 +156,9 @@ class InputServer(StubServerMixin):
         ss = self.get_server_source(proto)
         if ss is None:
             return
-        keyname = bytestostr(keyname)
-        try:
-            keystr = keystr.decode("utf8")
-        except Exception:
-            keystr = bytestostr(keystr)
-        modifiers = list(bytestostr(x) for x in modifiers)
+        keyname = net_utf8(keyname)
+        keystr = net_utf8(keystr)
+        modifiers = list(net_utf8(x) for x in modifiers)
         self.set_ui_driver(ss)
         keycode, group = self.get_keycode(ss, client_keycode, keyname, pressed, modifiers, keyval, keystr, group)
         keylog("process_key_action(%s) server keycode=%s, group=%i", packet, keycode, group)
@@ -183,7 +184,7 @@ class InputServer(StubServerMixin):
         return ss.get_keycode(client_keycode, keyname, pressed, modifiers, keyval, keystr, group)
 
     def fake_key(self, keycode, press):
-        pass
+        keylog("fake_key%s is not implemented", (keycode, press))
 
     def _handle_key(self, wid, pressed, name, keyval, keycode, modifiers, is_mod=False, sync=True):
         """
@@ -237,12 +238,13 @@ class InputServer(StubServerMixin):
         if pressed:
             delay_ms = min(1500, max(250, delay_ms))
             keylog("scheduling key repeat timer with delay %s for %s / %s", delay_ms, keyname, keycode)
-            now = monotonic_time()
-            self.key_repeat_timer = self.timeout_add(delay_ms, self._key_repeat_timeout, now, delay_ms, wid, keyname, keyval, keycode, modifiers, is_mod)
+            now = monotonic()
+            self.key_repeat_timer = self.timeout_add(delay_ms, self._key_repeat_timeout,
+                                                     now, delay_ms, wid, keyname, keyval, keycode, modifiers, is_mod)
 
     def _key_repeat_timeout(self, when, delay_ms, wid, keyname, keyval, keycode, modifiers, is_mod):
         self.key_repeat_timer = None
-        now = monotonic_time()
+        now = monotonic()
         keylog("key repeat timeout for %s / '%s' - clearing it, now=%s, scheduled at %s with delay=%s",
                keyname, keycode, now, when, delay_ms)
         self._handle_key(wid, False, keyname, keyval, keycode, modifiers, is_mod, True)
@@ -275,7 +277,7 @@ class InputServer(StubServerMixin):
             when_timedout = self.keys_timedout.get(keycode, None)
             if when_timedout:
                 del self.keys_timedout[keycode]
-            now = monotonic_time()
+            now = monotonic()
             if when_timedout and (now-when_timedout)<30:
                 #not so long ago, just re-press it now:
                 keylog("key %s/%s, had timed out, re-pressing it", keycode, keyname)
@@ -301,16 +303,17 @@ class InputServer(StubServerMixin):
                     ss.keys_changed()
 
     def clear_keys_pressed(self):
-        pass
+        keylog("clear_keys_pressed() is not implemented")
 
-    def get_keyboard_config(self, _props=None):
+    def get_keyboard_config(self, props=None):
+        keylog("get_keyboard_config(%s) is not implemented", props)
         return None
 
     def set_keyboard_repeat(self, key_repeat):
-        pass
+        keylog("set_keyboard_repeat(%s)", key_repeat)
 
     def set_keymap(self, ss, force=False):
-        pass
+        keylog("set_keymap(%s, %s)", ss, force)
 
 
     ######################################################################
@@ -350,7 +353,7 @@ class InputServer(StubServerMixin):
         return None
 
     def do_process_mouse_common(self, proto, wid, pointer, *args):
-        return pointer
+        return True
 
     def _process_button_action(self, proto, packet):
         mouselog("process_button_action(%s, %s)", proto, packet)
@@ -360,15 +363,16 @@ class InputServer(StubServerMixin):
         if ss is None:
             return
         ss.user_event()
+        self.last_mouse_user = ss.uuid
         self.set_ui_driver(ss)
         self.do_process_button_action(proto, *packet[1:])
 
     def do_process_button_action(self, proto, wid, button, pressed, pointer, modifiers, *args):
-        pass
+        """ all servers should implement this method """
 
 
     def _update_modifiers(self, proto, wid, modifiers):
-        pass
+        """ servers subclasses may change the modifiers state """
 
     def _process_pointer_position(self, proto, packet):
         mouselog("_process_pointer_position(%s, %s) readonly=%s, ui_driver=%s",
@@ -386,6 +390,7 @@ class InputServer(StubServerMixin):
         if self.ui_driver and self.ui_driver!=ss.uuid:
             return
         ss.user_event()
+        self.last_mouse_user = ss.uuid
         if self._process_mouse_common(proto, wid, pdata, *packet[5:]):
             self._update_modifiers(proto, wid, modifiers)
 
