@@ -7,6 +7,7 @@
 import os
 import sys
 import time
+from time import monotonic
 from multiprocessing import Queue as MQueue, freeze_support #@UnresolvedImport
 from gi.repository import GLib
 
@@ -17,7 +18,7 @@ from xpra.util import (
 from xpra.os_util import (
     get_username_for_uid, get_groups, get_home_for_uid, bytestostr,
     getuid, getgid, WIN32, POSIX, OSX,
-    monotonic_time, umask_context, get_group_id,
+    umask_context, get_group_id,
     )
 from xpra.net.socket_util import SOCKET_DIR_MODE, SOCKET_DIR_GROUP
 from xpra.server.server_core import ServerCore
@@ -71,7 +72,7 @@ class ProxyServer(ServerCore):
 
     def __init__(self):
         log("ProxyServer.__init__()")
-        ServerCore.__init__(self)
+        super().__init__()
         self._max_connections = MAX_CONCURRENT_CONNECTIONS
         self._start_sessions = False
         self.session_type = "proxy"
@@ -147,7 +148,7 @@ class ProxyServer(ServerCore):
                 log.error(" %s", e)
 
     def init_control_commands(self):
-        ServerCore.init_control_commands(self)
+        super().init_control_commands()
         self.control_commands["stop"] = ArgsControlCommand("stop", "stops the proxy instance on the given display",
                                                            self.handle_stop_command, min_args=1, max_args=1)
 
@@ -164,7 +165,7 @@ class ProxyServer(ServerCore):
 
 
     def init_packet_handlers(self):
-        ServerCore.init_packet_handlers(self)
+        super().init_packet_handlers()
         #add shutdown handler
         self._default_packet_handlers["shutdown-server"] = self._process_proxy_shutdown_server
 
@@ -181,7 +182,10 @@ class ProxyServer(ServerCore):
         return "proxy"
 
     def init_aliases(self):
-        pass
+        """
+        It is a lot less confusing if proxy servers don't use packet aliases at all.
+        So we override the aliases initialization and skip it.
+        """
 
     def do_run(self):
         self.main_loop = GLib.MainLoop()
@@ -235,10 +239,10 @@ class ProxyServer(ServerCore):
     def cleanup(self):
         self.stop_all_proxies()
         super().cleanup()
-        start = monotonic_time()
+        start = monotonic()
         live = True
         log("cleanup() proxy instances: %s", self.instances)
-        while monotonic_time()-start<PROXY_CLEANUP_GRACE_PERIOD and live:
+        while monotonic()-start<PROXY_CLEANUP_GRACE_PERIOD and live:
             live = tuple(x for x in tuple(self.instances.keys()) if x.is_alive())
             if live:
                 log("cleanup() still %i proxies alive: %s", len(live), live)
@@ -412,7 +416,7 @@ class ProxyServer(ServerCore):
                 return
             if display:
                 if display not in displays:
-                    if (":%s" % display) in displays:
+                    if ":%s" % display in displays:
                         display = ":%s" % display
                     else:
                         disconnect(SESSION_NOT_FOUND, "display '%s' not found" % display)
@@ -474,11 +478,13 @@ class ProxyServer(ServerCore):
             return
         log("server connection=%s", server_conn)
 
-        cipher = None
+        cipher = cipher_mode = None
         encryption_key = None
         if auth_caps:
             cipher = auth_caps.get("cipher")
             if cipher:
+                from xpra.net.crypto import DEFAULT_MODE
+                cipher_mode = auth_caps.get("cipher.mode", DEFAULT_MODE)
                 encryption_key = self.get_encryption_key(client_proto.authenticators, client_proto.keyfile)
 
         use_thread = PROXY_INSTANCE_THREADED
@@ -497,7 +503,7 @@ class ProxyServer(ServerCore):
             from xpra.server.proxy.proxy_instance_thread import ProxyInstanceThread
             pit = ProxyInstanceThread(session_options, self.video_encoders, self.pings,
                                       client_proto, server_conn,
-                                      disp_desc, cipher, encryption_key, c)
+                                      disp_desc, cipher, cipher_mode, encryption_key, c)
             pit.stopped = self.reap
             pit.run()
             self.instances[pit] = (False, display, None)
@@ -531,7 +537,7 @@ class ProxyServer(ServerCore):
                 process = ProxyInstanceProcess(uid, gid, env_options, session_options, self._socket_dir,
                                                self.video_encoders, self.pings,
                                                client_conn, disp_desc, client_state,
-                                               cipher, encryption_key, server_conn, c, message_queue)
+                                               cipher, cipher_mode, encryption_key, server_conn, c, message_queue)
                 log("starting %s from pid=%s", process, os.getpid())
                 self.instances[process] = (True, display, message_queue)
                 process.start()
