@@ -4,14 +4,14 @@
 # later version. See the file COPYING for details.
 
 import os
-import re
 
 #ensure that we use gtk as display source:
 from xpra.x11.gtk_x11.gdk_display_source import init_gdk_display_source
-from xpra.util import std, csv, envbool
+from xpra.util import std, csv, envbool, typedict
 from xpra.os_util import bytestostr
 from xpra.gtk_common.error import xsync, xlog
 from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings #@UnresolvedImport
+from xpra.keyboard.layouts import parse_xkbmap_query
 from xpra.log import Logger
 
 init_gdk_display_source()
@@ -59,7 +59,7 @@ def clean_keyboard_state():
 # keyboard layouts
 
 def do_set_keymap(xkbmap_layout, xkbmap_variant, xkbmap_options,
-                  xkbmap_print, xkbmap_query, xkbmap_query_struct={}):
+                  xkbmap_query, xkbmap_query_struct):
     """ xkbmap_layout is the generic layout name (used on non posix platforms)
         xkbmap_variant is the layout variant (may not be set)
         xkbmap_print is the output of "setxkbmap -print" on the client
@@ -71,46 +71,32 @@ def do_set_keymap(xkbmap_layout, xkbmap_variant, xkbmap_options,
     #First we try to use data from setxkbmap -query,
     #preferably as structured data:
     if xkbmap_query and not xkbmap_query_struct:
-        from xpra.keyboard.layouts import parse_xkbmap_query
         xkbmap_query_struct = parse_xkbmap_query(xkbmap_query)
+    xkbmap_query_struct = typedict(xkbmap_query_struct)
     if xkbmap_query_struct:
         log("do_set_keymap using xkbmap_query struct=%s", xkbmap_query_struct)
-        """ The xkbmap_query_struct data will look something like this:
-            {
-            b"rules"       : b"evdev",
-            b"model"       : b"pc105",
-            b"layout"      : b"gb",
-            b"options"     : b"grp:shift_caps_toggle",
-            }
-        """
+        #The xkbmap_query_struct data will look something like this:
+        #    {
+        #    b"rules"       : b"evdev",
+        #    b"model"       : b"pc105",
+        #    b"layout"      : b"gb",
+        #    b"options"     : b"grp:shift_caps_toggle",
+        #    }
         #parse the data into a dict:
-        rules = xkbmap_query_struct.get(b"rules")
-        model = xkbmap_query_struct.get(b"model")
-        layout = xkbmap_query_struct.get(b"layout")
-        variant = xkbmap_query_struct.get(b"variant")
-        options = xkbmap_query_struct.get(b"options")
+        rules = xkbmap_query_struct.strget("rules")
+        model = xkbmap_query_struct.strget("model")
+        layout = xkbmap_query_struct.strget("layout")
+        variant = xkbmap_query_struct.strget("variant")
+        options = xkbmap_query_struct.strget("options")
         if layout:
             log.info("setting keymap: %s",
                      csv("%s=%s" % (std(k), std(v)) for k,v in xkbmap_query_struct.items()
-                         if k in ("rules", "model", "layout") and v))
-            safe_setxkbmap(rules, model, layout, variant, options)
+                         if k in ("rules", "model", "layout", "variant", "options") and v))
+            if safe_setxkbmap(rules, model, layout, variant, options):
+                return
         else:
-            safe_setxkbmap(rules, model, "", "", "")
-    if xkbmap_print:
-        #TODO: this is no longer used with any client, remove it
-        log("do_set_keymap using xkbmap_print")
-        #try to guess the layout by parsing "setxkbmap -print"
-        try:
-            sym_re = re.compile(r"\s*xkb_symbols\s*{\s*include\s*\"([\w\+]*)")
-            for line in xkbmap_print.splitlines():
-                m = sym_re.match(line)
-                if m:
-                    layout = std(m.group(1))
-                    log.info("guessing keyboard layout='%s'" % layout)
-                    safe_setxkbmap("evdev", "pc105", layout, "", xkbmap_options)
-                    return
-        except Exception as e:
-            log.info("error setting keymap: %s" % e)
+            if safe_setxkbmap(rules, model, "", "", ""):
+                return
     #fallback for non X11 clients:
     layout = xkbmap_layout or "us"
     log.info("setting keyboard layout to '%s'", std(layout))
@@ -120,7 +106,7 @@ def safe_setxkbmap(rules, model, layout, variant, options):
     #(we execute the options separately in case that fails..)
     try:
         X11Keyboard.setxkbmap(rules, model, layout, variant, options)
-        return
+        return True
     except Exception:
         log("safe_setxkbmap%s", (rules, model, layout, variant, options), exc_info=True)
         log.warn("Warning: failed to set exact keymap,")
@@ -130,11 +116,12 @@ def safe_setxkbmap(rules, model, layout, variant, options):
         #try again with no options:
         try:
             X11Keyboard.setxkbmap(rules, model, layout, variant, "")
+            return True
         except Exception:
             log("setxkbmap", exc_info=True)
             log.error("Error: failed to set exact keymap")
             log.error(" even without applying any options..")
-
+    return False
 
 ################################################################################
 # keycodes
@@ -350,7 +337,10 @@ def set_all_keycodes(xkbmap_x11_keycodes, xkbmap_keycodes, preserve_server_keyco
     for try_harder in (False, True):
         filtered_preserve_keycode_entries = filter_mappings(indexed_mappings(preserve_keycode_entries), try_harder)
         log("filtered_preserve_keycode_entries=%s", filtered_preserve_keycode_entries)
-        trans, new_keycodes, missing_keycodes = translate_keycodes(kcmin, kcmax, keycodes, filtered_preserve_keycode_entries, keysym_to_modifier, try_harder)
+        trans, new_keycodes, missing_keycodes = translate_keycodes(kcmin, kcmax, keycodes,
+                                                                   filtered_preserve_keycode_entries,
+                                                                   keysym_to_modifier,
+                                                                   try_harder)
         if not missing_keycodes:
             break
     instructions = keymap_to_xmodmap(new_keycodes)
@@ -423,7 +413,7 @@ def x11_keycodes_to_list(x11_mappings):
     return entries
 
 
-def translate_keycodes(kcmin, kcmax, keycodes, preserve_keycode_entries={}, keysym_to_modifier={}, try_harder=False):
+def translate_keycodes(kcmin, kcmax, keycodes, preserve_keycode_entries, keysym_to_modifier, try_harder=False):
     """
         The keycodes given may not match the range that the server supports,
         or some of those keycodes may not be usable (only one modifier can
@@ -738,7 +728,7 @@ def get_modifiers_from_keycodes(xkbmap_keycodes, add_default_modifiers=True):
         if modifier:
             keynames = matches.setdefault(modifier, [])
             if keyname not in keynames:
-                keynames.add(keyname)
+                keynames.append(keyname)
             all_keynames.add(keyname)
     if add_default_modifiers:
         #try to add missings ones (magic!)
